@@ -43,6 +43,11 @@ class AdditionalText(models.Model):
         return u'Text "{0}" for {1}'.format(trunc(self.text), self.ipn.code)
 
 
+class BOMManager(models.Manager):
+    def get_queryset(self):
+        return super(BOMManager, self).get_queryset().select_related('ipn')
+
+
 class BOM(models.Model):
     """
     Bill Of Materials.
@@ -66,14 +71,22 @@ class BOM(models.Model):
         blank=True, null=True,
     )
 
+    objects = BOMManager()
+
     def __unicode__(self):
-        if self.ipn is not None:
+        if self.ipn_id is not None:
             ipn_code = self.ipn.code
         else:
             ipn_code = None
         return u'BOM "{0}" for {1}'.format(trunc(self.description), ipn_code)
 
 
+class BOMItemManager(models.Manager):
+    def get_queryset(self):
+        return super(BOMItemManager, self).get_queryset().select_related(
+            'ipn', 'bom', 'unit')
+
+
 class BOMItem(models.Model):
     """
     Details about an item, that belongs to a BOM.
@@ -112,12 +125,20 @@ class BOMItem(models.Model):
         related_name='bomitems',
     )
 
+    objects = BOMItemManager()
+
     def __unicode__(self):
         return u'{0} {1} of item "{2}" of BOM "{3}"'.format(
             self.qty, self.unit.code, self.ipn.code,
             trunc(self.bom.description))
 
 
+class CBOMManager(models.Manager):
+    def get_queryset(self):
+        return super(CBOMManager, self).get_queryset().select_related(
+            'customer')
+
+
 class CBOM(models.Model):
     """
     Customer Bill Of Materials.
@@ -158,6 +179,8 @@ class CBOM(models.Model):
         verbose_name=_('Version date'),
     )
 
+    objects = CBOMManager()
+
     def __unicode__(self):
         return u'{0} from "{1}" ({2})'.format(
             self.product, self.customer.description, self.version_date)
@@ -252,6 +275,12 @@ class CBOM(models.Model):
         return os.path.join(settings.MEDIA_URL, file_name)
 
 
+class CBOMItemManager(models.Manager):
+    def get_queryset(self):
+        return super(CBOMItemManager, self).get_queryset().select_related(
+            'bom', 'epn', 'epn__company', 'unit')
+
+
 class CBOMItem(models.Model):
     """
     Details about the items, that belong to a CBOM.
@@ -298,9 +327,23 @@ class CBOMItem(models.Model):
         related_name='cbomitems',
     )
 
+    # fields for caching the related values to increase performance
+    unit_code = models.CharField(max_length=5, blank=True)
+    epn_code = models.CharField(max_length=513, blank=True)
+    bom_short_description = models.CharField(max_length=128, blank=True)
+
+    objects = CBOMItemManager()
+
     def __unicode__(self):
         return u'{0} {1} of item "{2}" of CBOM "{3}"'.format(
-            self.qty, self.unit.code, self.epn, trunc(self.bom.description))
+            self.qty, self.unit_code, self.epn_code,
+            self.bom_short_description)
+
+    def save(self, *args, **kwargs):
+        self.unit_code = self.unit.code
+        self.epn_code = self.epn.__unicode__()
+        self.bom_short_description = trunc(self.bom.description)
+        return super(CBOMItem, self).save(*args, **kwargs)
 
 
 class Company(models.Model):
@@ -361,6 +404,12 @@ class Country(models.Model):
         return self.description
 
 
+class EPNManager(models.Manager):
+    def get_queryset(self):
+        return super(EPNManager, self).get_queryset().select_related(
+            'company', 'ipn', 'cpn')
+
+
 class EPN(models.Model):
     """
     External Part Number.
@@ -405,13 +454,29 @@ class EPN(models.Model):
         blank=True, null=True,
     )
 
+    # caching fields only
+    company_description = models.CharField(max_length=256, blank=True)
+    ipn_code = models.CharField(max_length=256, blank=True)
+
+    objects = EPNManager()
+
     def __unicode__(self):
-        if self.ipn is not None:
-            code = self.ipn.code
-        else:
-            code = None
         return u'EPN "{0}" from "{1}" (IPN: {2})'.format(
-            self.epn, self.company.description, code)
+            self.epn, self.company_description, self.ipn_code)
+
+    def save(self, *args, **kwargs):
+        if self.ipn_id is not None:
+            self.ipn_code = self.ipn.code
+        else:
+            self.ipn_code = ''
+        self.company_description = self.company.description
+        return super(EPN, self).save(*args, **kwargs)
+
+
+class IPNManager(models.Model):
+    def get_queryset(self):
+        return super(IPNManager, self).get_queryset().select_related(
+            'price_group', 'shape')
 
 
 class IPN(models.Model):
@@ -460,6 +525,8 @@ class IPN(models.Model):
         related_name='ipns',
     )
 
+    objects = IPNManager()
+
     @property
     def price_max(self):
         try:
